﻿using System;
using Telerik.Windows.Controls.Scheduler;
using Telerik.Windows.Controls.Scheduler.ICalendar;

namespace LinqToSqlProvider.ObjectModel
{
    /// <summary>
    /// Represents a single appointment object used by RadScheduler.
    /// </summary>
    public partial class SqlAppointment : IAppointment
    {
        private RecurrenceRule recurrenceRule;

        /// <summary>
        /// Occurs when <see cref="RecurrenceRule"/> status is changed.
        /// </summary>
        public event EventHandler RecurrenceRuleChanged;

        /// <summary>
        /// Gets or sets the recurrence rule.
        /// </summary>
        /// <value>The recurrence rule.</value>
        public RecurrenceRule RecurrenceRule
        {
            get
            {
                if (this.recurrenceRule == null)
                {
                    RecurrencePattern recurrencePattern;
                    if (CalHelper.TryParseRecurrencePattern(this.RecurrencePattern, out recurrencePattern))
                    {
                        this.RecurrenceRule = new RecurrenceRule(recurrencePattern);
                    }
                }
                return this.recurrenceRule;
            }
            set
            {
                if (this.recurrenceRule != value)
                {
                    if (this.recurrenceRule != null)
                    {
                        this.recurrenceRule.StateChanged -= this.OnRecurrenceRuleStateChanged;
                    }
                    this.recurrenceRule = value;
                    if (this.recurrenceRule != null)
                    {
                        this.recurrenceRule.StateChanged += this.OnRecurrenceRuleStateChanged;
                        TimeSpan duration = this.End - this.Start;
                        DateTime? next = this.recurrenceRule.Pattern.GetFirstOccurrence(this.Start);
                        if (next != null)
                        {
                            this.Start = next.Value;
                            this.End = this.Start + duration;
                        }
                    }
                    this.SendPropertyChanged("RecurrenceRule");
                }
            }
        }

        /// <summary>
        /// Gets or sets the appointment time zone.
        /// </summary>
        /// <value>The appointment time zone.</value>
        public TimeZoneInfo TimeZone
        {
            get
            {
                if (!String.IsNullOrEmpty(this.TimeZoneString))
                {
                    return TimeZoneInfo.FromSerializedString(this.TimeZoneString);
                }
                return null;
            }
            set
            {
                this.TimeZoneString = value.ToSerializedString();
            }
        }

        /// <summary>
        /// <b>Deep</b> copies this instance.
        /// </summary>
        /// <returns>A <b>deep</b> copy of the current object.</returns>
        public IAppointment Copy()
        {
            IAppointment appointment = new SqlAppointment();
            appointment.CopyFrom(this);
            return appointment;
        }

        /// <summary>
        /// <b>Deep</b> copies all properties from <paramref name="other"/> to this <see cref="IAppointment"/>.
        /// </summary>
        /// <param name="other">The <see cref="IAppointment"/> which properties are copied.</param>
        public void CopyFrom(IAppointment other)
        {
            this.Start = other.Start;
            this.End = other.End;
            this.IsAllDayEvent = other.IsAllDayEvent;
            this.Subject = other.Subject;

            if (other.RecurrenceRule == null)
            {
                this.RecurrenceRule = null;
            }
            else
            {
                if (this.RecurrenceRule == null)
                {
                    this.RecurrenceRule = other.RecurrenceRule.Copy();
                }
                else
                {
                    this.RecurrenceRule.CopyFrom(other.RecurrenceRule);
                }
            }

            var otherAppointment = other as SqlAppointment;
            if (otherAppointment == null)
                return;

            this.Body = otherAppointment.Body;
            this.UniqueId = otherAppointment.UniqueId;
            this.Url = otherAppointment.Url;
            this.Location = otherAppointment.Location;
        }

        protected virtual void OnRecurrenceRuleChanged(EventArgs args)
        {
            if (this.RecurrenceRuleChanged != null)
            {
                this.RecurrenceRuleChanged(this, args);
            }
        }

        partial void OnCreated()
        {
            this.UniqueId = Guid.NewGuid();
            if (this.RecurrenceRule != null)
            {
                this.Type = this.RecurrenceRule.Exceptions == null
                                ? (int)AppointmentType.Regular
                                : (int)AppointmentType.Exception;
            }
            else
            {
                this.Type = (int)AppointmentType.Regular;
            }
        }

        private void OnRecurrenceRuleStateChanged(object sender, EventArgs e)
        {
            this.OnRecurrenceRuleChanged(EventArgs.Empty);
        }
    }
}